use strict;
use warnings;
use WWW::Selenium;
use Alien::SeleniumRC;
use t::Bugzilla;

use constant TIMEOUT_SECONDS => 20;
use constant COLOR           => 1;
my $CTRL_M = COLOR ? "\r" : "\n";

my $T     = t::Bugzilla->new;
my $notes = $T->notes;

my $FIREFOX_BIN = '/usr/lib/firefox-1.5.0.10/firefox-bin';

my %SELENIUM_ARGS = (
    host        => $notes->{host},
    browser     => "*firefox $FIREFOX_BIN",
    browser_url => $notes->{browser_url},
);

my $jarfile = Alien::SeleniumRC::find_jar_location();
my $cmd     = "java -jar $jarfile";

my $pid_file = 'selenium-server.pid';
my $pid      = fork();
if ( not defined $pid ) {
    print "can not fork the selenium server. resources are not avilable.\n";
}
elsif ( $pid == 0 ) {
    print "Starting The Selenium Server\n";
    exec($cmd) or die "couldn’t exec `$cmd`: $!";
}
else {

    open PID_FILE, ">$pid_file"
      or die "couldn't open $pid_file";
    print PID_FILE "$pid";

    print "$pid\n";
    $pid = fork();
    wait_till_is_up(TIMEOUT_SECONDS);
}

exit 0;

#############################################################################
# wait till the server is up and return 1
# if the waiting times out returns 0
sub wait_till_is_up {
    my ($timeout) = @_;
    my $sleep_interval = 1;    # secs

    my $sel       = WWW::Selenium->new(%SELENIUM_ARGS);
    my $server_up = sub {

        # start returns a session id. on DESTROY will auto stop
        my $id = eval { $sel->start };
        return !$@ && defined $id ? 1 : 0;
    };

    if ( $server_up->() ) {
        return 1;
    }

    my $start_time = time;
    my $preamble   = "${CTRL_M}still waiting for server to warm up: ";
    print $preamble unless COLOR;
    while (1) {
        my $delta = time - $start_time;
        print COLOR
          ? ( $preamble, sprintf "%02d:%02d", ( gmtime $delta )[ 1, 0 ] )
          : '.';
        sleep $sleep_interval;
        if ( $server_up->() ) {
            print
              "${CTRL_M}the server is up (waited $delta secs)             \n";
            return 1;
        }
        elsif ( $delta > $timeout ) {
            print "${CTRL_M}the server is down, giving up after $delta secs\n";
            return 0;
        }
        else {

            # continue
        }
    }
}

